@@ -21,7 +21,6 @@ Z* -------------------------------------------------------------------
 #include "Base.h"
 #include "Seq.h"
 #include "main.h"
-#include "ScrollBar.h"
 #include "MemoryDebug.h"
 #include "PyMOLObject.h"
 #include "Scene.h"
@@ -34,27 +33,6 @@ Z* -------------------------------------------------------------------
 #include "Ortho.h"
 #include "CGO.h"
 
-struct _CSeq {
-  ::Block *Block;
-  int DragFlag;
-  int ScrollBarActive;
-  int NSkip;
-  struct CScrollBar *ScrollBar;
-  CSeqRow *Row;
-  int NRow;
-  int Size;
-  int VisSize;
-  int Changed;
-  int Dirty;
-  int LineHeight;
-  int CharWidth;
-  int ScrollBarWidth;
-  int ScrollBarMargin;
-  int CharMargin;
-  int LastRow;
-  CSeqHandler *Handler;         /* borrowed pointer */
-};
-
 static int SeqFindRowCol(PyMOLGlobals * G, int x, int y, int *row_num_ptr,
                          int *col_num_ptr, int fixed_row)
 {
@@ -69,14 +47,14 @@ static int SeqFindRowCol(PyMOLGlobals * G, int x, int y, int *row_num_ptr,
   if(fixed_row >= 0) {
     row_num = fixed_row;
   } else {
-    row_num = (y - I->Block->rect.bottom) / DIP2PIXEL(I->LineHeight);
+    row_num = (y - I->rect.bottom) / DIP2PIXEL(I->LineHeight);
     row_num = (I->NRow - 1) - row_num;
   }
   if((row_num >= 0) && (row_num < I->NRow)) {
     int char_num;
     CSeqRow *row;
-    row = I->Row + row_num;
-    char_num = (x - I->Block->rect.left - DIP2PIXEL(I->CharMargin)) / DIP2PIXEL(I->CharWidth);
+    row = I->Row.data() + row_num;
+    char_num = (x - I->rect.left - DIP2PIXEL(I->CharMargin)) / DIP2PIXEL(I->CharWidth);
     if(row->nCol && !row->label_flag)
       if(char_num < I->VisSize) {
         char_num += I->NSkip;
@@ -118,17 +96,16 @@ void SeqUpdate(PyMOLGlobals * G)
     OrthoReshape(G, -1, -1, false);     /* careful, this is recursive... */
   }
   if(I->Dirty) {
-    if(I->Handler->fRefresh)
-      I->Handler->fRefresh(G, I->Row);
+    I->Handler->refresh(G, I->Row);
     I->Dirty = false;
   }
 }
 
-static void SeqReshape(Block * block, int width, int height)
+void CSeq::reshape(int width, int height)
 {
-  PyMOLGlobals *G = block->G;
+  PyMOLGlobals *G = m_G;
   CSeq *I = G->Seq;
-  BlockReshape(block, width, height);
+  Block::reshape(width, height);
 
   {                             /* get current sequence sizes */
     int a;
@@ -141,8 +118,8 @@ static void SeqReshape(Block * block, int width, int height)
 
   {
     int extra;
-    I->VisSize = (I->Block->rect.right - I->Block->rect.left - 1) / DIP2PIXEL(I->CharWidth);
-    /*    printf("%d %d %d %d %d\n",cw,I->Block->rect.right,I->Block->rect.left,I->VisSize,I->Size); */
+    I->VisSize = (I->rect.right - I->rect.left - 1) / DIP2PIXEL(I->CharWidth);
+    /*    printf("%d %d %d %d %d\n",cw,I->rect.right,I->rect.left,I->VisSize,I->Size); */
 
     if(I->VisSize < 1)
       I->VisSize = 1;
@@ -151,7 +128,7 @@ static void SeqReshape(Block * block, int width, int height)
       I->ScrollBarActive = false;
     } else {
       I->ScrollBarActive = true;
-      ScrollBarSetLimits(I->ScrollBar, I->Size, I->VisSize);
+      m_ScrollBar.setLimits(I->Size, I->VisSize);
     }
   }
 }
@@ -170,9 +147,9 @@ void SeqChanged(PyMOLGlobals * G)
   SceneInvalidate(G);
 }
 
-static int SeqDrag(Block * block, int x, int y, int mod)
+int CSeq::drag(int x, int y, int mod)
 {
-  PyMOLGlobals *G = block->G;
+  PyMOLGlobals *G = m_G;
   CSeq *I = G->Seq;
   int pass = 0;
   int row_num;
@@ -180,40 +157,28 @@ static int SeqDrag(Block * block, int x, int y, int mod)
   if(!pass) {
     if(SeqFindRowCol(G, x, y, &row_num, &col_num, I->LastRow)) {
       if(I->Handler)
-        if(I->Handler->fDrag)
-          I->Handler->fDrag(G, I->Row, row_num, col_num, mod);
+        I->Handler->drag(G, I->Row, row_num, col_num, mod);
       OrthoDirty(G);
     }
   }
   return (1);
 }
 
-static int SeqRelease(Block * block, int button, int x, int y, int mod)
+int CSeq::release(int button, int x, int y, int mod)
 {
-  PyMOLGlobals *G = block->G;
+  PyMOLGlobals *G = m_G;
   CSeq *I = G->Seq;
   int pass = 0;
-  /*
-     if(I->ScrollBarActive) {
-     if((y-I->Block->rect.bottom)<I->ScrollBarWidth) {
-     pass = 1;
-     ScrollBarDoRelease(I->ScrollBar,button,x,y,mod);
-     OrthoUngrab(G);
-     }
-     } 
-   */
   if(!pass) {
     int row_num;
     int col_num;
     if(SeqFindRowCol(G, x, y, &row_num, &col_num, I->LastRow)) {
       if(I->Handler)
-        if(I->Handler->fRelease)
-          I->Handler->fRelease(G, I->Row, button, row_num, col_num, mod);
+        I->Handler->release(G, I->Row, button, row_num, col_num, mod);
       OrthoDirty(G);
     } else {
       if(I->Handler)
-        if(I->Handler->fRelease)
-          I->Handler->fRelease(G, I->Row, button, -1, -1, mod);
+        I->Handler->release(G, I->Row, button, -1, -1, mod);
       OrthoDirty(G);
     }
   }
@@ -241,24 +206,33 @@ void SeqSetHandler(PyMOLGlobals * G, CSeqHandler * handler)
   I->Handler = handler;
 }
 
-static int SeqClick(Block * block, int button, int x, int y, int mod)
+int CSeq::click(int button, int x, int y, int mod)
 {
-  PyMOLGlobals *G = block->G;
+  PyMOLGlobals *G = m_G;
   CSeq *I = G->Seq;
   int pass = 0;
   int row_num;
   int col_num;
+
+  switch(button) {
+    case P_GLUT_BUTTON_SCROLL_FORWARD:
+      I->m_ScrollBar.moveBy(-1);
+      return 1;
+    case P_GLUT_BUTTON_SCROLL_BACKWARD:
+      I->m_ScrollBar.moveBy(1);
+      return 1;
+  }
+
   if(I->ScrollBarActive) {
-    if((y - I->Block->rect.bottom) < DIP2PIXEL(I->ScrollBarWidth)) {
+    if((y - rect.bottom) < DIP2PIXEL(I->ScrollBarWidth)) {
       pass = 1;
-      ScrollBarDoClick(I->ScrollBar, button, x, y, mod);
+      I->m_ScrollBar.click(button, x, y, mod);
     }
   }
   if(!pass) {
     if(SeqFindRowCol(G, x, y, &row_num, &col_num, -1)) {
       if(I->Handler)
-        if(I->Handler->fClick)
-          I->Handler->fClick(G, I->Row, button, row_num, col_num, mod, x, y);
+        I->Handler->click(G, I->Row, button, row_num, col_num, mod, x, y);
       I->DragFlag = true;
       I->LastRow = row_num;
       OrthoDirty(G);
@@ -274,8 +248,7 @@ static int SeqClick(Block * block, int button, int x, int y, int mod)
         break;
       case P_GLUT_LEFT_BUTTON:
         if(I->Handler)
-          if(I->Handler->fClick)
-            I->Handler->fClick(G, I->Row, button, -1, -1, mod, x, y);
+          I->Handler->click(G, I->Row, button, -1, -1, mod, x, y);
         break;
       }
     }
@@ -283,15 +256,15 @@ static int SeqClick(Block * block, int button, int x, int y, int mod)
   return (1);
 }
 
-static void SeqDraw(Block * block ORTHOCGOARG)
+void CSeq::draw(CGO* orthoCGO)
 {
-  PyMOLGlobals *G = block->G;
+  PyMOLGlobals *G = m_G;
   CSeq *I = G->Seq;
 
   if(G->HaveGUI && G->ValidContext) {
 
-    int x = I->Block->rect.left;
-    int y = I->Block->rect.bottom + DIP2PIXEL(I->ScrollBarMargin) + 1;
+    int x = rect.left;
+    int y = rect.bottom + DIP2PIXEL(I->ScrollBarMargin) + 1;
     float bg_color[3] = { 0.f, 0.f, 0.f }, overlay_color[3] = { 1.0F, 1.0F, 1.0F };
     int label_color_index = SettingGetGlobal_color(G, cSetting_seq_view_label_color);
     const float *label_color = ColorGet(G, label_color_index);
@@ -312,16 +285,16 @@ static void SeqDraw(Block * block ORTHOCGOARG)
       } else {
 	glColor3fv(bg_color);
       }
-      BlockFill(I->Block ORTHOCGOARGVAR);
+      fill(orthoCGO);
     }
     if(I->ScrollBarActive) {
-      ScrollBarSetBox(I->ScrollBar, I->Block->rect.bottom + DIP2PIXEL(I->ScrollBarWidth),
-                      I->Block->rect.left + DIP2PIXEL(I->ScrollBarMargin),
-                      I->Block->rect.bottom + DIP2PIXEL(2),
-                      I->Block->rect.right - DIP2PIXEL(I->ScrollBarMargin));
-      ScrollBarDoDraw(I->ScrollBar ORTHOCGOARGVAR);
+      I->m_ScrollBar.setBox(rect.bottom + DIP2PIXEL(I->ScrollBarWidth),
+                            rect.left + DIP2PIXEL(I->ScrollBarMargin),
+                            rect.bottom + DIP2PIXEL(2),
+                            rect.right - DIP2PIXEL(I->ScrollBarMargin));
+      I->m_ScrollBar.draw(orthoCGO);
       y += DIP2PIXEL(I->ScrollBarWidth);
-      I->NSkip = (int) ScrollBarGetValue(I->ScrollBar);
+      I->NSkip = static_cast<int>(I->m_ScrollBar.getValue());
     } else {
       I->NSkip = 0;
     }
@@ -360,7 +333,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
       }
       fill_color = ColorGet(G, fill_color_index);
       if(unaligned_color_index < 0) {
-        unaligned_color = NULL;
+        unaligned_color = nullptr;
       } else {
         unaligned_color = ColorGet(G, unaligned_color_index);
       }
@@ -374,8 +347,8 @@ static void SeqDraw(Block * block ORTHOCGOARG)
       /* measure titles */
 
       for(a = I->NRow - 1; a >= 0; a--) {
-        row = I->Row + a;
-        col = row->col;
+        row = I->Row.data() + a;
+        col = row->col.data();
         if(row->label_flag || row->column_label_flag) {
           row->title_width = col->offset + (col->stop - col->start);
           if(max_title_width < row->title_width)
@@ -388,9 +361,9 @@ static void SeqDraw(Block * block ORTHOCGOARG)
       cur_color = overlay_color;
       TextSetColor(G, cur_color);
       for(a = I->NRow - 1; a >= 0; a--) {
-        row = I->Row + a;
+        row = I->Row.data() + a;
         yy = y1 - 2;
-        col = row->col;
+        col = row->col.data();
         if((row->label_flag || row->column_label_flag) && row->nCol) {
           row->title_width = col->offset + (col->stop - col->start);
           xx =
@@ -400,7 +373,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
           pix_wid = DIP2PIXEL(I->CharWidth * ch_wid);
           tot_len = col->offset + ch_wid - I->NSkip;
           if(tot_len <= vis_size) {
-            TextDrawSubStrFast(G, row->txt, xx, y1, col->start, ch_wid ORTHOCGOARGVAR);
+            TextDrawSubStrFast(G, row->txt, xx, y1, col->start, ch_wid, orthoCGO);
           }
         }
         y1 += DIP2PIXEL(I->LineHeight);
@@ -408,7 +381,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
 
       y1 = y;
       for(a = I->NRow - 1; a >= 0; a--) {
-        row = I->Row + a;
+        row = I->Row.data() + a;
         cur_color = overlay_color;
 	if (orthoCGO){
 	  CGOColorv(orthoCGO, cur_color);
@@ -507,7 +480,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
 		}
                 TextSetColor(G, black);
               }
-              TextDrawSubStrFast(G, row->txt, xx, y1, col->start, ch_wid ORTHOCGOARGVAR);
+              TextDrawSubStrFast(G, row->txt, xx, y1, col->start, ch_wid, orthoCGO);
             }
           }
         }
@@ -525,7 +498,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
               pix_wid = DIP2PIXEL(I->CharWidth * ch_wid);
               tot_len = col->offset + ch_wid - I->NSkip;
               if(tot_len <= vis_size) {
-                TextDrawCharRepeat(G, fill_char, xx, y1, col->start, ch_wid ORTHOCGOARGVAR);
+                TextDrawCharRepeat(G, fill_char, xx, y1, col->start, ch_wid, orthoCGO);
               }
             }
           }
@@ -544,7 +517,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
             stop_col = start_col;
             start_col = tmp;
           }
-          row = I->Row + box_row;
+          row = I->Row.data() + box_row;
           if((start_col >= 0) && (start_col < row->nCol) &&
              (stop_col >= 0) && (stop_col < row->nCol)) {
             int xx2;
@@ -591,7 +564,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
       if(I->ScrollBarActive) {
         int real_count = n_real;
         int mode = 0;
-        float width = (float) (I->Block->rect.right - I->Block->rect.left);
+        float width = (float) (rect.right - rect.left);
         float start = 0, stop;
         int right = 0;
         float bot, top, cent;
@@ -599,13 +572,13 @@ static void SeqDraw(Block * block ORTHOCGOARG)
         int last_color = -1;
         cur_color = blue;
         for(a = 0; a < I->NRow; a++) {
-          row = I->Row + a;
+          row = I->Row.data() + a;
           if(!row->label_flag) {
             top =
-              I->Block->rect.bottom + DIP2PIXEL(I->ScrollBarMargin) + (height * real_count) / n_real;
+              rect.bottom + DIP2PIXEL(I->ScrollBarMargin) + (height * real_count) / n_real;
             real_count--;
             bot =
-              I->Block->rect.bottom + DIP2PIXEL(I->ScrollBarMargin) + (height * real_count) / n_real;
+              rect.bottom + DIP2PIXEL(I->ScrollBarMargin) + (height * real_count) / n_real;
             mode = 0;
             for(b = 0; b < row->nCol; b++) {
               col = row->col + b;
@@ -719,7 +692,7 @@ static void SeqDraw(Block * block ORTHOCGOARG)
           }
         }
 
-        ScrollBarDrawHandle(I->ScrollBar, 0.35F ORTHOCGOARGVAR);
+        I->m_ScrollBar.drawHandle(0.35F, orthoCGO);
       }
     }
   }
@@ -727,33 +700,15 @@ static void SeqDraw(Block * block ORTHOCGOARG)
 
 int SeqInit(PyMOLGlobals * G)
 {
-  CSeq *I = NULL;
-  if((I = (G->Seq = Calloc(CSeq, 1)))) {
+  CSeq *I = nullptr;
+  if((I = (G->Seq = new CSeq(G)))) {
 
-    I->Block = OrthoNewBlock(G, NULL);
-    I->Block->fClick = SeqClick;
-    I->Block->fDraw = SeqDraw;
-    I->Block->fDrag = SeqDrag;
-    I->Block->fRelease = SeqRelease;
-    I->Block->fReshape = SeqReshape;
-    I->Block->active = true;
-    I->Block->TextColor[0] = 1.0;
-    I->Block->TextColor[1] = 0.75;
-    I->Block->TextColor[2] = 0.75;
-    OrthoAttach(G, I->Block, cOrthoTool);
-    I->DragFlag = false;
-    I->ScrollBarActive = true;
-    I->ScrollBar = ScrollBarNew(G, true);
-    ScrollBarSetValue(I->ScrollBar, 0);
-    I->Row = NULL;
-    I->NRow = 0;
-    I->Dirty = true;
-    I->ScrollBarWidth = 16;
-    I->ScrollBarMargin = 2;
-    I->LineHeight = 13;
-    I->CharMargin = 2;
-    I->LastRow = -1;
-    I->CharWidth = 8;
+    I->active = true;
+    I->TextColor[0] = 1.0;
+    I->TextColor[1] = 0.75;
+    I->TextColor[2] = 0.75;
+    OrthoAttach(G, I, cOrthoTool);
+    I->m_ScrollBar.setValue(0);
     return 1;
   } else
     return 0;
@@ -761,45 +716,25 @@ int SeqInit(PyMOLGlobals * G)
 
 static void SeqPurgeRowVLA(PyMOLGlobals * G)
 {
-  CSeq *I = G->Seq;
-  if(I->Row) {
-    int a;
-    CSeqRow *row;
-    for(a = 0; a < I->NRow; a++) {
-      row = I->Row + a;
-      VLAFreeP(row->txt);
-      VLAFreeP(row->col);
-      VLAFreeP(row->fill);
-      VLAFreeP(row->char2col);
-      VLAFreeP(row->atom_lists);
-    }
-    VLAFreeP(I->Row);
-  }
+  G->Seq->Row.clear();
 }
 
-void SeqSetRowVLA(PyMOLGlobals * G, CSeqRow * row, int nRow)
+void SeqSetRow(PyMOLGlobals * G, std::vector<CSeqRow>&& row, int nRow)
 {
   CSeq *I = G->Seq;
-  SeqPurgeRowVLA(G);
-  I->Row = row;
+  I->Row = std::move(row);
   I->NRow = nRow;
 }
 
 void SeqFree(PyMOLGlobals * G)
 {
-  CSeq *I = G->Seq;
-
-  SeqPurgeRowVLA(G);
-  if(I->ScrollBar)
-    ScrollBarFree(I->ScrollBar);
-  OrthoFreeBlock(G, I->Block);
-  FreeP(G->Seq);
+  DeleteP(G->Seq);
 }
 
 Block *SeqGetBlock(PyMOLGlobals * G)
 {
   CSeq *I = G->Seq;
   {
-    return (I->Block);
+    return (I);
   }
 }
